@@ -337,6 +337,14 @@
      target:[UpdateController sharedInstance]
      argument:self order:10 modes:NSDefaultRunLoopMode];*/
 #endif
+
+#ifdef GNUSTEP_BASE_VERSION
+    /* menu in window */
+    if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == NSWindows95InterfaceStyle)
+    {
+        [self new:self];
+    }
+#endif
 }
 
 
