@@ -1,11 +1,11 @@
 /* VGroup.m
  * Group of graphic objects
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2010 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2005-09-23
+ * modified: 2010-07-08 (relocated setList:)
  *
  * the group is implemented as follows:
  * - all objects are stored in a list with absolute coordinates
@@ -125,19 +125,29 @@
 
 - (NSString*)title		{ return @"Group"; }
 
-- list
+- (NSMutableArray*)list
 {
     return list;
 }
+- (void)setList:(NSArray*)aList
+{
+    [list release];
+    list = [aList retain];
+    [self setColorNew];
+    coordBounds = bounds = NSZeroRect;
+    dirty = YES;
+}
 
 - (unsigned)countRecursive
-{   int	i, cnt = [list count];
+{   int	i, cnt = 0;
 
     for (i=[list count]-1; i>=0; i--)
     {	id	g = [list objectAtIndex:i];
 
         if ( [g isKindOfClass:[VGroup class]] ) // || [g isKindOfClass:[VPath class]] 
             cnt += [g countRecursive];
+        else
+            cnt++;
     }
     return cnt;
 }
@@ -508,17 +518,6 @@
     [super setSelected:flag];
 }
 
-/* returns the old list
- */
-- (void)setList:(NSArray*)aList
-{
-    [list release];
-    list = [aList retain];
-    [self setColorNew];
-    coordBounds = bounds = NSZeroRect;
-    dirty = YES;
-}
-
 - (void)addObject:g
 {
     [list addObject:g];
@@ -611,19 +610,17 @@
  * add a list to our list
  * after the operation we are selected, all of our objects are deselected
  */
-- (void)add:addList
+- (void)add:(NSArray*)addList
 {   int	i, cnt;
 
     if (!list)
         list = [[NSMutableArray allocWithZone:[self zone]] init];
-
     for (i=0, cnt=[addList count]; i<cnt; i++)
     {	id	g = [addList objectAtIndex:i];
 
         [g setSelected:NO];
         [list addObject:g];
     }
-
     [self setSelected:YES];
     coordBounds = bounds = NSZeroRect;
 }
@@ -722,7 +719,8 @@
         coordBounds = [[list objectAtIndex:cnt] coordBounds];
         for (i=cnt-1; i>=0; i--)
         {   rect = [[list objectAtIndex:i] coordBounds];
-            coordBounds = VHFUnionRect(rect, coordBounds);
+            if ( rect.size.width > 0.0 && rect.size.height > 0.0 )    // in case we have a bogus object
+                coordBounds = VHFUnionRect(rect, coordBounds);
         }
     }
     return coordBounds;
@@ -795,8 +793,8 @@
                 //[obj drawKnobs:rect direct:direct scaleFactor:scaleFactor];
                 [obj drawControls:rect direct:direct scaleFactor:scaleFactor];
                 [obj setSelected:sel];
-/*                if ([obj isMemberOfClass:[VCurve class]])
-                {   [NSBezierPath setDefaultLineWidth:0.0];
+                /*if ([obj isMemberOfClass:[VCurve class]])
+                {   [NSBezierPath setDefaultLineWidth:1.0/scaleFactor];
                     [NSBezierPath strokeLineFromPoint:[obj pointWithNum:0] toPoint:[obj pointWithNum:1]];
                     [NSBezierPath strokeLineFromPoint:[obj pointWithNum:3] toPoint:[obj pointWithNum:2]];
                 }*/
@@ -1065,6 +1063,15 @@
     dirty = YES;
 }
 
+- (void)transform:(NSAffineTransform*)matrix
+{   int	i;
+
+    for ( i=[list count]-1; i >= 0; i-- )
+        [[list objectAtIndex:i] transform:matrix];
+    coordBounds = bounds = NSZeroRect;
+    dirty = YES;
+}
+
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)cp
 {   int		i;
 
