@@ -1,10 +1,11 @@
 /* VImage.m
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1998-03-22
- * modified: 2010-04-14 (-initWithRepresentations: 10.6 keep image size despite using rep size)
+ * modified: 2012-01-30 (-drawWithPrincipal: if principal is nil -> set scale = 1.0)
+ *           2010-04-14 (-initWithRepresentations: 10.6 keep image size despite using rep size)
  *           2010-02-25 (-initWithRepresentations: workaround for Mac OS 10.6 problem with image sizes < pixel size)
  *           2010-01-29 (-writeFilesToDirectory: modified type converion to work with Mac OS 10.6)
  *           2008-12-01 (-drawWithPrincipal: relief draw pale)
@@ -1081,7 +1082,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 {   NSPoint         p;
     NSSize          currentSize;
     NSRect          bounds, rect;
-    float           scale = [principal scaleFactor];
+    float           scale = (principal) ? [principal scaleFactor] : 1.0;
     NSBezierPath    *bPath;
     BOOL            releaseImage = NO;
     NSImage         *sepImage = nil; // color separated image
@@ -1090,7 +1091,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         return;
 
     bounds.origin = origin;
-    bounds.size = size;
+    bounds.size   = size;
     bounds.size.width  *= scale;
     bounds.size.height *= scale;
 
@@ -1991,9 +1992,11 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 #           endif
         }
         else    // GIF, JPG, PNG, ...
-        {   NSBitmapImageRep    *rep = [[image representations] objectAtIndex:0];
+        {   NSArray             *reps = [image representations];
+            NSBitmapImageRep    *rep = ([reps count]) ? [reps objectAtIndex:0] : nil;
+
 #           ifdef __APPLE__ // workaround NSCGImageSnapshotRep (drag into doc)
-            if ( ! [rep isKindOfClass:[NSBitmapImageRep class]] )
+            if ( !rep || ! [rep isKindOfClass:[NSBitmapImageRep class]] )
             {   data = [image TIFFRepresentationUsingCompression:NSTIFFCompressionNone factor:0.0];
                 rep = [NSBitmapImageRep imageRepWithData:data];
             }
