@@ -1,11 +1,15 @@
 /* VText.m
  * 2-D Text object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-02-15
- * modified: 2009-12-29 (-drawWithPrincipal:): for GNUstep renew drawText after rotAngle
+ * modified: 2012-02-28 (-drawWithPrincipal: set frame twice to make mirrored text visible)
+ *           2012-02-28 (-textDidEndEditing: if locked, we don't adjust the size of the text box)
+ *           2010-07-29 (-drawWithPrincipal:...) move -setTextContainerInset: before -setFrame to make mirror on Apple visible
+ *           2010-07-28 (+valueForKey:inArray:) new
+ *           2009-12-29 (-drawWithPrincipal:): for GNUstep renew drawText after rotAngle
  *           2009-02-12 (-textDidEndEditing: add correction to height to make text appear safely on Apple)
  *           2009-02-11 (-getFlattenedObjectAt: set yOffset for mirrored text)
  *           2009-01-27 (-textDidEndEditing: remove empty text)
@@ -117,6 +121,26 @@ static void initClassVars()
     return;
 }
 
+/* search given array of Graphics objects
+ * and return the substring (value) of a text with a given key
+ * ex: "#KEY#Returned String"
+ * created: 2010-07-28
+ */
++ (NSString*)valueForKey:(NSString*)key inArray:(NSArray*)array
+{   int         i, cnt = [array count];
+
+    for ( i=0; i<cnt; i++ )
+    {   VText   *g = [array objectAtIndex:i];
+
+        if ( [g isKindOfClass:[VText class]] )
+        {   NSString    *string = [g string];
+
+            if ( [string hasPrefix:key] )
+                return [string substringFromIndex:[key length]];
+        }
+    }
+    return nil;
+}
 
 - (void)renewSharedText
 {
@@ -651,21 +675,18 @@ static void initClassVars()
 }
 
 - (void)editorFrameChanged:(NSNotification *)arg
-{
-    NSRect currentEditingFrame = [[arg object] frame];
+{   NSRect  currentEditingFrame = [[arg object] frame];
 
     if (!NSEqualRects(lastEditingFrame, NSZeroRect))
     {
         if (lastEditingFrame.size.width > currentEditingFrame.size.width)
-        {
-            NSRect updateRect = lastEditingFrame;
+        {   NSRect  updateRect = lastEditingFrame;
 
             updateRect.origin.x = currentEditingFrame.origin.x + currentEditingFrame.size.width;
             [self updateEditingViewRect:updateRect];
         }
         if (lastEditingFrame.size.height > currentEditingFrame.size.height)
-        {
-            NSRect updateRect = lastEditingFrame;
+        {   NSRect updateRect = lastEditingFrame;
 
             updateRect.origin.y = currentEditingFrame.origin.y + currentEditingFrame.size.height;
             [self updateEditingViewRect:updateRect];
@@ -1044,8 +1065,8 @@ static void initClassVars()
             int		i, cnt = [mas length];
 
             for (i=0; i<cnt; i++)
-            {   NSRange			range;
-                NSColor			*colorAtt;
+            {   NSRange range;
+                NSColor *colorAtt;
 
                 colorAtt = [mas attribute:@"NSColor" atIndex:i longestEffectiveRange:&range
                                   inRange:NSMakeRange(0, cnt)];
@@ -1064,11 +1085,12 @@ static void initClassVars()
             [[drawText textStorage] setAttributedString:attributedString];
 
         [[drawText textStorage] endEditing];
+        [drawText setTextContainerInset:NSMakeSize(0.0, 0.0)];  // 2010-07-29: moved before setFrame: for Apple mirror
         [drawText setFrame:bounds];
+        [drawText setFrame:bounds]; // 2012-02-28: well, without this extra invitation Apple doesn't draw mirrored text !
         //[drawText writeRTFDToFile:@"/tmp/text.rtf" atomically:NO];	// debugging
         //[drawText setFrame:bounds];
 
-        [drawText setTextContainerInset:NSMakeSize(0.0, 0.0)];
         if ( centerVertical )	// vertical center: (bounds.size.height-textHeight)/2
         {   NSRect	rect = [lm boundingRectForGlyphRange:NSMakeRange(0, [lm numberOfGlyphs])
                                              inTextContainer:[drawText textContainer]];
@@ -1086,7 +1108,7 @@ static void initClassVars()
         [drawText removeFromSuperview];
         [window setAutodisplay:saveAutoDisplay];
         [drawText scaleUnitSquareToSize:NSMakeSize(1.0, 1.0/aspectRatio)];
-#ifdef GNUSTEP_BASE_VERSION // 2009-12-29 gui 0.17.1: GNUstpe workaround, rotation screws up everything
+#ifdef GNUSTEP_BASE_VERSION // 2009-12-29 gui 0.17.1: GNUstep workaround, rotation screws up everything
         if ( rotAngle != 0.0 )
         {   [drawText release]; drawText = nil;
             initClassVars();
@@ -1528,7 +1550,8 @@ static void initClassVars()
  * back to nil.
  * For further explanation of the two "ack!" lines, see edit:in: above.
  *
- * modified: 2009-02-12
+ * modified: 2012-02-28 (if isLocked, we don't adjust the size of the text box)
+ *           2009-02-12
  */
 - (void)textDidEndEditing:(NSNotification *)notification
 {   NSTextView          *fe = [notification object];
@@ -1564,12 +1587,14 @@ static void initClassVars()
 
             oldBounds = [self extendedBoundsWithScale:[graphicView scaleFactor]];
             [fe setVerticallyResizable:YES];
-            [fe sizeToFit];
-            bounds = [fe frame];
-            bounds.size = [fe bounds].size;
-            bounds.size.width  += correction;
-            bounds.size.height += correction;   // origin = upper/left here!
-            bounds = [editView convertRect:bounds toView:graphicView];  // ack!
+            if ( ! isLocked )
+            {   [fe sizeToFit];
+                bounds = [fe frame];
+                bounds.size = [fe bounds].size;
+                bounds.size.width  += correction;
+                bounds.size.height += correction;   // origin = upper/left here!
+                bounds = [editView convertRect:bounds toView:graphicView];  // ack!
+            }
             //bounds = [graphicView centerScanRect:bounds];	/* center pixels */
             //redrawRect = NSUnionRect(bounds, redrawRect);
 
@@ -1685,7 +1710,7 @@ static void initClassVars()
             [pathG setAngle:-rotAngle withCenter:bRect.origin];
             /* FIXME: stroked fonts shouldn't be filled! */
             [pathG setFilled:YES optimize:NO];
-            if (group)
+            if ( group && [pathG count] )
                 [group addObject:pathG];
         }
 
