@@ -429,7 +429,8 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             price = nil;
             if ( priceDict )
-            {   NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
+            {   //NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
+                NSString    *v = ([iArray count]) ? [iArray objectAtIndex:0] : appVersion;  // version of installed module
                 NSRange     range = [v rangeOfString:@"." options:NSBackwardsSearch];
 
                 if ( range.length )
@@ -814,7 +815,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             for ( i=0, path=nil; i<[downloadFiles count]; i++)
             {   path = [[downloadFiles objectAtIndex:i] objectAtIndex:1];
-                if ( [fileManager fileExistsAtPath:path] )  // delete TAR-file
+                if ( [[path pathExtension] isEqual:@"tar"] && [fileManager fileExistsAtPath:path] ) // del TAR-file
                     [fileManager removeFileAtPath:path handler:nil];
                 // TODO: the following must not happen before the files are really installed !
                 /*path = [path stringByDeletingPathExtension];
